#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''AS Configurations v1 action implementations'''
import argparse
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def set_attributes_for_print_detail(obj):
    info = {}
    info['name'] = obj.name
    info['id'] = obj.id
    info['create_time'] = obj.create_time
    info['type'] = obj.type
    info['status'] = obj.status
    info['scaling_group_id'] = obj.scaling_group_id
    info['alarm_id'] = obj.alarm_id
    info['cool_down_time'] = obj.cool_down_time
    info['scheduled_policy'] = obj.scheduled_policy
    info['scaling_policy_action'] = obj.scaling_policy_action

    return info


class ListAutoScalingPolicy(command.Lister):
    _description = _('List AutoScaling Policies')
    columns = ('ID', 'Name')

    def get_parser(self, prog_name):
        parser = super(ListAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            # dest='limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results displayed')
        )
        parser.add_argument(
            '--marker',
            # dest='marker',
            metavar='<ID>',
            help=_('Begin displaying the results for IDs greater than the '
                   'specified marker. When used with --limit, set this to '
                   'the last ID displayed in the previous run')
        )
        parser.add_argument(
            '--group',
            metavar='<group>',
            required=True,
            help=_('ScalingGroup ID or Name')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        args = {}
        if parsed_args.limit:
            args['limit'] = parsed_args.limit
        if parsed_args.marker:
            args['marker'] = parsed_args.marker

        group = client.find_group(parsed_args.group, ignore_missing=False)

        data = client.policies(group=group.id, **args)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowAutoScalingPolicy(command.ShowOne):
    _description = _('Shows details of a AutoScalinig policy')
    columns = ['ID', 'Name', 'scaling_group_id', 'status',
               'type', 'alarm_id', 'scheduled_policy',
               'scaling_policy_action', 'cool_down_time']

    def get_parser(self, prog_name):
        parser = super(ShowAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('ID of the configuration policy\n'
                    'For Policy Name search --group param is needed')
        )
        parser.add_argument(
            '--group',
            metavar='<group>',
            help=_('ScalingGroup ID or Name if Name searched is used')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        if parsed_args.group:
            group = client.find_group(parsed_args.group, ignore_missing=False)
            obj = client.find_policy(parsed_args.policy,
                                     group=group,
                                     ignore_missing=False)
        else:
            obj = client.get_policy(parsed_args.policy)

        fmt = set_attributes_for_print_detail(obj)
        # display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(
            fmt, self.columns, formatters={})

        return (self.columns, data)


class CreateAutoScalingPolicy(command.ShowOne):
    _description = _('Creates AutoScalinig Policy')
    columns = ['ID', 'Name', 'scaling_group_id', 'status',
               'type', 'alarm_id', 'scheduled_policy',
               'scaling_policy_action', 'cool_down_time'
               ]
#     columns = ['ID', 'Name', 'instance_id', 'instance_name',
#                'flavor_id', 'image_id', 'disk',
#                'key_name', 'public_ip', 'user_data', 'metadata'
#                ]
#
    POLICY_TYPES = ['ALARM', 'SCHEDULED', 'RECURRENCE']

    def get_parser(self, prog_name):
        parser = super(CreateAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('AS Policy name')
        )
        parser.add_argument(
            '--group',
            metavar='<group>',
            required=True,
            help=_('AS Group ID or Name for the AS Policy')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            required=True,
            # choices=['ALARM', 'SCHEDULED', 'RECURRENCE'],
            help=_('AS Policy type [`ALARM`, `SCHEDULED`, `RECURRENCE`]')
        )
        parser.add_argument(
            '--cool-down-time',
            metavar='<cool_down_time>',
            type=int,
            help=_('Specifies the cooling time in seconds for the policy')
        )
        parser.add_argument(
            '--alarm-id',
            metavar='<alarm_id>',
            help=_('Specifies the alarm_id for the policy')
        )
        parser.add_argument(
            '--action-operation',
            metavar='<action_operation>',
            help=_('Specifies the policy operation '
                   'Can be [`ADD`, `REMOVE`, `SET`]')
        )
        parser.add_argument(
            '--action-instance-number',
            metavar='<action_instance_number>',
            type=int,
            help=_('Specifies number of instances to be operated')
        )
        parser.add_argument(
            '--launch-time',
            metavar='<launch_time>',
            help=_('Specifies the time when the scaling action is triggered. '
                   'The time format must comply with UTC.\n'
                   '* when type=`SCHEDULED`, then `YYYY-MM-DDThh:mmZ`\n'
                   '* when type=`RECURRENCE`, then `hh:mm`\n')
        )
        parser.add_argument(
            '--recurrence-type',
            metavar='<recurrence_type>',
            help=_(
                'Specifies the periodic triggering type\n'
                'This parameter is mandatory when type=`RECURRENCE`\n'
                'Can be [`Daily`, `Weekly`, `Monthly`]'
            )
        )
        parser.add_argument(
            '--recurrence-value',
            metavar='<recurrence_value>',
            help=_(
                'Specifies the frequency, at which actions are triggered\n'
                'When recurrente_type=`Daily` it is Null\n'
                'When recurrente_type=`Weekly` it\'s a week day number '
                '[1..7], where 1 is for Sunday\n'
                'When recurrente_type=`Monthly` it\'s a day number [1..31]\n'
            )
        )
        parser.add_argument(
            '--start-time',
            metavar='<start_time>',
            help=_('Specifies the start time in of the action in the '
                   '`YYYY-MM-DDThh:mmZ` format')
        )
        parser.add_argument(
            '--end-time',
            metavar='<end_time>',
            help=_('Specifies the end time in of the action in the '
                   '`YYYY-MM-DDThh:mmZ` format\n'
                   'Mandatory when type=`RECURRENCE`')
        )
        return parser

    def take_action(self, parsed_args):

        policy_attrs = {}
        policy_attrs['name'] = parsed_args.name
        policy_attrs['scaling_group_id'] = parsed_args.group
        policy_type = parsed_args.type.upper()
        if policy_type not in self.POLICY_TYPES:
            msg = (_('Unsupported policy type. Should be one of %s')
                   % self.POLICY_TYPES)
            raise argparse.ArgumentTypeError(msg)
        else:
            policy_attrs['type'] = policy_type
        if parsed_args.alarm_id:
            policy_attrs['alarm_id'] = parsed_args.alarm_id
        if parsed_args.cool_down_time:
            policy_attrs['cool_down_time'] = parsed_args.cool_down_time
        policy_action = {}
        if parsed_args.action_operation:
            policy_action['operation'] = parsed_args.action_operation
        if parsed_args.action_instance_number:
            policy_action['instance_number'] = \
                parsed_args.action_instance_number
        if policy_action.keys():
            policy_attrs['scaling_policy_action'] = policy_action
        scheduled_policy = {}
        if parsed_args.launch_time:
            scheduled_policy['launch_time'] = parsed_args.launch_time
        if parsed_args.recurrence_type:
            # TODO(agoncharov) validate input
            scheduled_policy['recurrence_type'] = parsed_args.recurrence_type
        if parsed_args.recurrence_value:
            scheduled_policy['recurrence_value'] = parsed_args.recurrence_value
        if parsed_args.launch_time:
            scheduled_policy['start_time'] = parsed_args.start_time
        if parsed_args.launch_time:
            scheduled_policy['end_time'] = parsed_args.end_time
        if scheduled_policy.keys():
            policy_attrs['scheduled_policy'] = scheduled_policy

        client = self.app.client_manager.auto_scaling

        policy = client.create_policy(**policy_attrs)

        fmt = set_attributes_for_print_detail(policy)
        # display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(
            fmt, self.columns, formatters={})

        return (self.columns, data)


class DeleteAutoScalingPolicy(command.Command):
    _description = _('Deletes AutoScalinig policy')

    def get_parser(self, prog_name):
        parser = super(DeleteAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            nargs='+',
            metavar='<policy>',
            help=_('AS Policy ID')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        # TODO(agoncharov) - proper error handling (reporting)

        for pol in parsed_args.policy:
            client.delete_policy(pol)


class UpdateAutoScalingPolicy(command.ShowOne):
    _description = _('Updates AutoScalinig Policy')
    columns = ['ID', 'Name', 'scaling_group_id', 'status',
               'type', 'alarm_id', 'scheduled_policy',
               'scaling_policy_action', 'cool_down_time'
               ]
#     columns = ['ID', 'Name', 'instance_id', 'instance_name',
#                'flavor_id', 'image_id', 'disk',
#                'key_name', 'public_ip', 'user_data', 'metadata'
#                ]
#
    POLICY_TYPES = ['ALARM', 'SCHEDULED', 'RECURRENCE']

    def get_parser(self, prog_name):
        parser = super(UpdateAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('AS Policy ID')
        )
        parser.add_argument(
            '--group',
            metavar='<group>',
            required=True,
            help=_('AS Group ID or Name for the AS Policy')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_('AS Policy type [`ALARM`, `SCHEDULED`, `RECURRENCE`]')
        )
        parser.add_argument(
            '--cool-down-time',
            metavar='<cool_down_time>',
            type=int,
            help=_('Specifies the cooling time in seconds for the policy')
        )
        parser.add_argument(
            '--alarm-id',
            metavar='<alarm_id>',
            help=_('Specifies the alarm_id for the policy')
        )
        parser.add_argument(
            '--action-operation',
            metavar='<action_operation>',
            help=_('Specifies the policy operation '
                   'Can be [`ADD`, `REMOVE`, `SET`]')
        )
        parser.add_argument(
            '--action-instance-number',
            metavar='<action_instance_number>',
            type=int,
            help=_('Specifies number of instances to be operated')
        )
        parser.add_argument(
            '--launch-time',
            metavar='<launch_time>',
            help=_('Specifies the time when the scaling action is triggered. '
                   'The time format must comply with UTC.\n'
                   '* when type=`SCHEDULED`, then `YYYY-MM-DDThh:mmZ`\n'
                   '* when type=`RECURRENCE`, then `hh:mm`\n')
        )
        parser.add_argument(
            '--recurrence-type',
            metavar='<recurrence_type>',
            help=_(
                'Specifies the periodic triggering type\n'
                'This parameter is mandatory when type=`RECURRENCE`\n'
                'Can be [`Daily`, `Weekly`, `Monthly`]'
            )
        )
        parser.add_argument(
            '--recurrence-value',
            metavar='<recurrence_value>',
            help=_(
                'Specifies the frequency, at which actions are triggered\n'
                'When recurrente_type=`Daily` it is Null\n'
                'When recurrente_type=`Weekly` it is a week day number '
                '[1..7], where 1 is for Sunday\n'
                'When recurrente_type=`Monthly` it is a day number [1..31]\n'
            )
        )
        parser.add_argument(
            '--start-time',
            metavar='<start_time>',
            help=_('Specifies the start time in of the action in the '
                   '`YYYY-MM-DDThh:mmZ` format')
        )
        parser.add_argument(
            '--end-time',
            metavar='<end_time>',
            help=_('Specifies the end time in of the action in the '
                   '`YYYY-MM-DDThh:mmZ` format\n'
                   'Mandatory when type=`RECURRENCE`')
        )
        return parser

    def take_action(self, parsed_args):

        policy_attrs = {}
        # policy_attrs['name'] = parsed_args.name
        policy_attrs['scaling_group_id'] = parsed_args.group
        if parsed_args.type:
            policy_type = parsed_args.type.upper()
            if policy_type not in self.POLICY_TYPES:
                msg = (_('Unsupported policy type. Should be one of %s')
                       % self.POLICY_TYPES)
                raise argparse.ArgumentTypeError(msg)
            else:
                policy_attrs['type'] = policy_type
        if parsed_args.alarm_id:
            policy_attrs['alarm_id'] = parsed_args.alarm_id
        if parsed_args.cool_down_time:
            policy_attrs['cool_down_time'] = parsed_args.cool_down_time
        policy_action = {}
        if parsed_args.action_operation:
            policy_action['operation'] = parsed_args.action_operation
        if parsed_args.action_instance_number:
            policy_action['instance_number'] = \
                parsed_args.action_instance_number
        if policy_action.keys():
            policy_attrs['scaling_policy_action'] = policy_action
        scheduled_policy = {}
        if parsed_args.launch_time:
            scheduled_policy['launch_time'] = parsed_args.launch_time
        if parsed_args.recurrence_type:
            # TODO(agoncharov) validate input
            scheduled_policy['recurrence_type'] = parsed_args.recurrence_type
        if parsed_args.recurrence_value:
            scheduled_policy['recurrence_value'] = parsed_args.recurrence_value
        if parsed_args.launch_time:
            scheduled_policy['start_time'] = parsed_args.start_time
        if parsed_args.launch_time:
            scheduled_policy['end_time'] = parsed_args.end_time
        if scheduled_policy.keys():
            policy_attrs['scheduled_policy'] = scheduled_policy

        client = self.app.client_manager.auto_scaling

        policy = client.get_policy(parsed_args.policy)
        policy = client.update_policy(
            policy, **policy_attrs)

        fmt = set_attributes_for_print_detail(policy)
        # display_columns, columns = _get_columns(obj)
        data = utils.get_dict_properties(
            fmt, self.columns, formatters={})

        return (self.columns, data)


class ExecuteAutoScalingPolicy(command.Command):
    _description = _('Executes AutoScalinig policy')

    def get_parser(self, prog_name):
        parser = super(ExecuteAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('AS Policy ID or name')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        if parsed_args.policy:
            client.execute_policy(parsed_args.policy)


class EnableAutoScalingPolicy(command.Command):
    _description = _('Enables (resume) AutoScalinig policy')

    def get_parser(self, prog_name):
        parser = super(EnableAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('AS Policy ID')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        if parsed_args.policy:
            client.resume_policy(parsed_args.policy)


class DisableAutoScalingPolicy(command.Command):
    _description = _('Disables (pause) AutoScalinig policy')

    def get_parser(self, prog_name):
        parser = super(DisableAutoScalingPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('AS Policy ID')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.auto_scaling

        if parsed_args.policy:
            client.pause_policy(parsed_args.policy)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
# from unittest.mock import call

# from osc_lib import exceptions

from otcextensions.osclient.identity.v3 import credential
from otcextensions.tests.unit.osclient.identity.v3 import fakes

# from openstackclient.tests.unit import utils as tests_utils


class TestListIdentityCredentials(fakes.TestIdentity):

    objects = fakes.FakeIdentityCredential.create_multiple(3)

    column_list_headers = (
        'Access Key',
        'Description',
        'User ID',
        'Status',
        'Created At',
    )

    columns = (
        'access',
        'description',
        'user_id',
        'status',
        'created_at',
    )

    data = []

    for o in objects:
        data.append(
            (o.id, o.description, o.user_id, o.status, o.created_at))

    def setUp(self):
        super(TestListIdentityCredentials, self).setUp()

        self.cmd = credential.ListCredentials(self.app, None)

        self.client.credentials = mock.Mock()
        self.client.api_mock = self.client.credentials

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--user-id', '1'
        ]

        verifylist = [
            ('user_id', '1'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            user_id='1',
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import datetime
import random
import uuid

import mock

from otcextensions.sdk.cbr.v3 import backup
from otcextensions.sdk.cbr.v3 import checkpoint
from otcextensions.sdk.cbr.v3 import policy
from otcextensions.sdk.cbr.v3 import member
from otcextensions.sdk.cbr.v3 import task
from otcextensions.sdk.cbr.v3 import vault
from otcextensions.tests.unit.osclient import test_base


def generate_vault_list():
    """Generate random list of vault UUIDs"""
    vault_list = []
    random_int = random.randint(1, 10)
    while random_int > 0:
        vault_list.append({
            'vault_id': uuid.uuid4().hex})
        random_int -= 1
    return vault_list


def generate_pattern():
    pattern = ''
    on_daily_base = random.choice([True, False])
    days = ['MO', 'TU', 'WE', 'TH', 'FR', 'SA', 'SU']
    if on_daily_base:
        pattern = 'FREQ=DAILY;INTERVAL=' + str(random.randint(1, 30)) + ';'
    else:
        pattern = 'FREQ=WEEKLY;BYDAY='
        day_count = random.randint(1, 7)
        day_list = random.sample(days, day_count)
        day_list_len = len(day_list)
        for item in day_list:
            pattern = pattern + item
            if day_list_len != 1:
                pattern += ','
            else:
                pattern += ';'
            day_list_len -= 1
    byhour = random.randint(0, 23)
    if byhour < 10:
        byhour = 'BYHOUR=' + '0' + str(byhour) + ';'
    else:
        byhour = 'BYHOUR=' + str(byhour) + ';'
    pattern += byhour
    byminute = random.randint(0, 59)
    if byminute < 10:
        byminute = 'BYMINUTE=' + '0' + str(byminute)
    else:
        byminute = 'BYMINUTE=' + str(byminute)
    pattern += byminute
    return pattern


def generate_pattern_list():
    """Generate random list of patterns"""
    pattern_list = []
    random_int = random.randint(1, 5)
    while random_int > 0:
        pattern_list.append(generate_pattern())
        random_int -= 1
    return pattern_list


class TestCBR(test_base.TestCommand):

    def setUp(self):
        super(TestCBR, self).setUp()

        self.app.client_manager.cbr = mock.Mock()
        self.client = self.app.client_manager.cbr


class FakeBackup(test_base.Fake):
    """Fake one or more CBR backups"""

    @classmethod
    def generate(cls):
        object_info = {
            "id": 'id-' + uuid.uuid4().hex,
            "name": 'name-' + uuid.uuid4().hex,
            "checkpoint_id": 'id-' + uuid.uuid4().hex,
            "created_at": uuid.uuid4().hex,
            "description": uuid.uuid4().hex,
            "expired_at": uuid.uuid4().hex,
            "image_type": uuid.uuid4().hex,
            "parent_id": uuid.uuid4().hex,
            "project_id": uuid.uuid4().hex,
            "protected_at": uuid.uuid4().hex,
            "resource_az": uuid.uuid4().hex,
            "resource_id": uuid.uuid4().hex,
            "resource_name": 'resname' + uuid.uuid4().hex,
            "resource_size": 5,
            "resource_type": random.choice(['OS::Cinder::Volume',
                                            'OS::Nova::Server']),
            "status": 'available',
            "updated_at": uuid.uuid4().hex,
            "vault_id": uuid.uuid4().hex,
            "provider_id": uuid.uuid4().hex,
            "children": [{'id': 'child_backup_uuid_1'},
                         {'id': 'child_backup_uuid_2'}]
        }

        obj = backup.Backup.existing(**object_info)
        return obj


class FakeCheckpoint(test_base.Fake):
    """Fake one or more CBR checkpoint"""

    @classmethod
    def generate(cls):
        object_info = {
            'created_at': uuid.uuid4().hex,
            'id': 'pid-' + uuid.uuid4().hex,
            'status': 'available',
            'name': 'checkpoint-' + uuid.uuid4().hex,
            'vault':
                {
                    'id': uuid.uuid4().hex,
                    'name': uuid.uuid4().hex,
                    "resources": [
                        {
                            'name': 'resource-name-' + uuid.uuid4().hex,
                            'resource_size': '6',
                            'backup_size': '6840',
                            'protect_status': 'available',
                            'backup_count': '18',
                            'type': 'OS::Nova::Server',
                            'id': 'pid-' + uuid.uuid4().hex,
                            'extra_info': '{}'
                        }]
            },
            'extra_info':
                {
                    'name': 'backup-' + uuid.uuid4().hex
            }
        }

        obj = checkpoint.Checkpoint.existing(**object_info)
        return obj


class FakePolicy(test_base.Fake):
    """Fake one or more CBR policies with random vaults list and patterns"""

    @classmethod
    def generate(cls):
        object_info = {
            "name": 'name-' + uuid.uuid4().hex,
            "associated_vaults": generate_vault_list(),
            "enabled": random.choice([True, False]),
            "trigger": {
                "properties": {
                    "pattern": generate_pattern_list(),
                    "start_time": datetime.datetime.now(),
                },
                "type": "time",
                "id": 'trigger-' + "uuid.uuid4().hex",
                "name": "default"
            },
            "operation_definition": {
                "max_backups": random.randint(1, 99999),
                "retention_duration_days": random.randint(1, 99999),
                "year_backups": random.randint(0, 100),
                "day_backups": random.randint(0, 100),
                "month_backups": random.randint(0, 100),
                "week_backups": random.randint(0, 100),
                "timezone": 'UTC+0' + str(random.randint(0, 9)) + ':00',
            },
            "operation_type": random.choice(['backup', 'replication']),
            "id": 'id-' + uuid.uuid4().hex,
        }

        obj = policy.Policy.existing(**object_info)
        return obj


class FakePolicyFixed(test_base.Fake):
    """Fake one or more CBR policies with fixed associated vaults and
        schedule patterns.
    """

    @classmethod
    def generate(cls):
        object_info = {
            "name": 'name-' + uuid.uuid4().hex,
            "associated_vaults": [
                {'vault_id': 'vault_id_1'},
                {'vault_id': 'vault_id_2'},
                {'vault_id': 'vault_id_3'},
            ],
            "enabled": random.choice([True, False]),
            "trigger": {
                "properties": {
                    "pattern": [
                        'pattern_1',
                        'pattern_2'
                    ],
                    "start_time": datetime.datetime.now(),
                },
                "type": "time",
                "id": 'trigger-' + "uuid.uuid4().hex",
                "name": "default"
            },
            "operation_definition": {
                "max_backups": random.randint(1, 99999),
                "retention_duration_days": random.randint(1, 99999),
                "year_backups": random.randint(0, 100),
                "day_backups": random.randint(0, 100),
                "month_backups": random.randint(0, 100),
                "week_backups": random.randint(0, 100),
                "timezone": 'UTC+0' + str(random.randint(0, 9)) + ':00',
            },
            "operation_type": random.choice(['backup', 'replication']),
            "id": 'id-' + uuid.uuid4().hex,
        }

        obj = policy.Policy.existing(**object_info)
        return obj


class FakeMember(test_base.Fake):
    """Fake one or more CBR members"""

    @classmethod
    def generate(cls):
        object_info = {
            "id": 'id-' + uuid.uuid4().hex,
            "status": random.choice(['pending', 'accepted', 'rejected']),
            "created_at": uuid.uuid4().hex,
            "updated_at": uuid.uuid4().hex,
            "backup_id": uuid.uuid4().hex,
            "image_id": uuid.uuid4().hex,
            "dest_project_id": uuid.uuid4().hex,
            "vault_id": uuid.uuid4().hex
        }

        obj = member.Member.existing(**object_info)
        return obj


class FakeTask(test_base.Fake):
    """Fake one or more CBR task with random vaults list and patterns"""

    @classmethod
    def generate(cls):
        object_info = {
            'status': 'success',
            'provider_id': 'pid-' + uuid.uuid4().hex,
            'checkpoint_id': 'cid-' + uuid.uuid4().hex,
            'updated_at': uuid.uuid4().hex,
            'error_info': {'message': '', 'code': ''},
            'vault_id': 'pid-' + uuid.uuid4().hex,
            'started_at': uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'ended_at': uuid.uuid4().hex,
            'created_at': uuid.uuid4().hex,
            'operation_type': 'backup',
            'vault_name': 'vault-' + uuid.uuid4().hex,
            'project_id': 'prjid-' + uuid.uuid4().hex,
            'policy_id': 'polid-' + uuid.uuid4().hex
        }

        obj = task.Task.existing(**object_info)
        return obj


class FakeVault(test_base.Fake):
    """Fake one or more CBR vault with random vaults list and patterns"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'backup_policy_id': 'bid-' + uuid.uuid4().hex,
            'created_at': uuid.uuid4().hex,
            'provider_id': uuid.uuid4().hex,
            'user_id': uuid.uuid4().hex,
            'billing': {
                'cloud_type': 'public',
                'consistent_level': 'crash_consistent',
                'object_type': 'server',
                'protect_type': 'backup',
                'size': random.randint(0, 100),
                'charging_mode': 'post_paid',
                'is_auto_renew': False,
                'is_auto_pay': False,
            },
            'description': 'vault_description',
            'auto_bind': False,
            'name': 'vault-' + uuid.uuid4().hex,
            'resources': [{
                'extra_info': {
                    'include_volumes': [{
                        'id': 'vid-' + uuid.uuid4().hex,
                        'os_version': 'CentOS 7.6 64bit'
                    }]
                },
                'id': 'resource_id',
                'type': 'OS::Nova::Server'
            }],
            'tags': [{
                'key': 'key-tags',
                'value': 'val-tags'
            }],
            'bind_rules': {
                'tags': [{
                    'key': 'key-bind',
                    'value': 'val-bind'
                }]
            },
            'project_id': '0'
        }

        obj = vault.Vault.existing(**object_info)
        return obj


class VaultDefaultStruct:
    def __init__(self, **entries):
        self.__dict__.update(entries)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.sdk.ctsv3.v3 import key_event as _key_event
from otcextensions.sdk.ctsv3.v3 import trace as _trace
from otcextensions.sdk.ctsv3.v3 import tracker as _tracker
from otcextensions.sdk.ctsv3.v3 import quota as _quota


class Proxy(proxy.Proxy):
    skip_discovery = True

    def create_key_event(self, **attrs):
        """Create an event

        :param dict attrs: Keyword arguments which will be used to create a
             :class:`~otcextensions.sdk.ctsv3.v3.key_event.KeyEvent`
        :returns: The key event
        :rtype: :class:`~otcextensions.sdk.ctsv3.v3.key_event.KeyEvent`
         """
        return self._create(_key_event.KeyEvent, **attrs)

    def update_key_event(self, **attrs):
        """Update an event

        :param dict attrs: Keyword arguments which will be used to overwrite a
             :class:`~otcextensions.sdk.ctsv3.v3.key_event.KeyEvent`
        :returns: The updated key event
        :rtype: :class:`~otcextensions.sdk.ctsv3.v3.key_event.KeyEvent`
         """
        return self._update(_key_event.KeyEvent, "", **attrs)

    def delete_key_event(self, notification):
        """Delete an event

        :param notification: The key event to delete a
            :class:`~otcextensions.sdk.ctsv3.v3.key_event.KeyEvent`
        :returns: None
         """
        notification.delete_key(self)

    def key_events(self, notification_type, **attrs):
        """Query notification events

        :param notification_type The type of notification to query
        :returns: A generator of key event object of
        :class:`~otcextensions.sdk.ctsv3.v3.key_event.KeyEvent`
        """
        base_path = f'{_key_event.KeyEvent.base_path}/{notification_type}'
        return self._list(_key_event.KeyEvent,
                          base_path=base_path,
                          **attrs)

    def traces(self, **attrs):
        """Query traces

        :returns: A generator of trace object of a
            :class:`~otcextensions.sdk.ctsv3.v3.trace.Trace`
        """
        return self._list(_trace.Trace, paginated=False, **attrs)

    def trackers(self, **attrs):
        """Query notification events

        :param dict attrs: Optional query parameters to be sent to limit the
            resources being returned.
            * `tracker_name`: Tracker name

        :returns: A generator of tracker object of a
            :class:`~otcextensions.sdk.ctsv3.v3.tracker.Tracker`
        """
        return self._list(_tracker.Tracker, **attrs)

    def create_tracker(self, **attrs):
        """Create tracker

        :param dict attrs: Keyword arguments which will be used to create a
             :class:`~otcextensions.sdk.ctsv3.v3.tracker.Tracker`
        :returns: The key event
        :rtype: :class:`~otcextensions.sdk.ctsv3.v3.tracker.Tracker`
         """
        return self._create(
            _tracker.Tracker,
            base_path=_tracker.Tracker.base_path[:-1],
            **attrs)

    def delete_tracker(self, tracker):
        """Delete a single tracker

        :param tracker: The tracker to delete a
            :class:`~otcextensions.sdk.ctsv3.v3.tracker.Tracker`
        :returns: None
         """
        tracker.delete_tracker(self)

    def update_tracker(self, **attrs):
        """Update a single tracker

        :param dict attrs: Keyword arguments which will be used to overwrite a
            :class:`~otcextensions.sdk.ctsv3.v3.tracker.Tracker`
        :returns: None
        """
        self._update(_tracker.Tracker,
                     base_path=_tracker.Tracker.base_path[:-1],
                     **attrs)

    def quotas(self):
        """Query quotas

        :returns: A generator of quota object of a
            :class:`~otcextensions.sdk.ctsv3.v3.quota.Quota`
        """
        return self._list(_quota.Quota)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from openstack.tests.unit import test_proxy_base
from otcextensions.sdk.css.v1 import _proxy
from otcextensions.sdk.css.v1 import cluster as _cluster
from otcextensions.sdk.css.v1 import cluster_image as _cluster_image
from otcextensions.sdk.css.v1 import (
    cluster_upgrade_status as _cluster_upgrade_status
)
from otcextensions.sdk.css.v1 import flavor as _flavor
from otcextensions.sdk.css.v1 import snapshot as _snapshot

ENDPOINT_CSS = 'http://css.example.com/v1.0'


class TestCssProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestCssProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)
        self.session.get_endpoint = mock.Mock(return_value=ENDPOINT_CSS)

    def test_clusters(self):
        self.verify_list(
            self.proxy.clusters,
            _cluster.Cluster,
            method_kwargs={'id': 'foo', 'start': 999, 'limit': 666},
            expected_kwargs={'id': 'foo', 'start': 999, 'limit': 666},
        )

    def test_get_cluster(self):
        self.verify_get(self.proxy.get_cluster, _cluster.Cluster)

    def test_find_cluster(self):
        self.verify_find(self.proxy.find_cluster, _cluster.Cluster, False)

    def test_create_cluster(self):
        self.verify_create(
            self.proxy.create_cluster,
            _cluster.Cluster,
            method_kwargs={'x': 1, 'y': 2, 'z': 3},
            expected_kwargs={'x': 1, 'y': 2, 'z': 3},
        )

    def test_delete_cluster(self):
        self.verify_delete(self.proxy.delete_cluster, _cluster.Cluster, True)

    def test_restart_cluster(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.restart',
            self.proxy.restart_cluster,
            method_args=[_cluster.Cluster],
            expected_args=[self.proxy],
        )

    def test_extend_cluster(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.extend',
            self.proxy.extend_cluster,
            method_args=[_cluster.Cluster, 2],
            expected_args=[self.proxy, 2],
        )

    def test_extend_cluster_nodes(self):
        self.verify_create(
            self.proxy.extend_cluster_nodes,
            _cluster.ExtendClusterNodes,
            method_kwargs={
                'cluster': 'cluster-uuid',
                'grow': [{'x': 1, 'y': 2, 'z': 3}],
            },
            expected_kwargs={
                'cluster_id': 'cluster-uuid',
                'grow': [{'x': 1, 'y': 2, 'z': 3}],
            },
        )

    def test_update_cluster_name(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_name',
            self.proxy.update_cluster_name,
            method_args=[_cluster.Cluster, 'test'],
            expected_args=[self.proxy, 'test'],
        )

    def test_update_password(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_password',
            self.proxy.update_cluster_password,
            method_args=[_cluster.Cluster, 'password'],
            expected_args=[self.proxy, 'password'],
        )

    def test_update_security_mode(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_security_mode',
            self.proxy.update_cluster_security_mode,
            method_args=[_cluster.Cluster, True, False, None],
            expected_args=[self.proxy, True, False, None],
        )

        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_security_mode',
            self.proxy.update_cluster_security_mode,
            method_kwargs={
                'cluster': _cluster.Cluster,
                'https_enable': False,
                'authority_enable': False,
                'admin_pwd': None,

            },
            expected_args=[self.proxy, False, False, None],
        )

    def test_update_security_group(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_security_group',
            self.proxy.update_cluster_security_group,
            method_args=[_cluster.Cluster, 'security-group.id'],
            expected_args=[self.proxy, 'security-group.id'],
        )

    def test_update_cluster_flavor(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_flavor',
            self.proxy.update_cluster_flavor,
            method_kwargs={
                'cluster': _cluster.Cluster,
                'new_flavor': 'test',
                'check_replica': True,
            },
            expected_args=[self.proxy, 'test', None, True],
        )

        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_flavor',
            self.proxy.update_cluster_flavor,
            method_args=[_cluster.Cluster, 'test', 'ess', True],
            expected_args=[self.proxy, 'test', 'ess', True],
        )

    def test_update_kernel(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.update_kernel',
            self.proxy.update_cluster_kernel,
            method_args=[
                _cluster.Cluster,
                'target-image-id',
                'upgrade-type',
                True,
                'agency',
                False,
            ],
            expected_args=[
                self.proxy,
                'target-image-id',
                'upgrade-type',
                True,
                'agency',
                False,
            ],
        )

    def test_get_cluster_version_upgrades(self):
        self.verify_get(
            self.proxy.get_cluster_version_upgrades,
            _cluster_image.ClusterImage,
            method_args=[],
            method_kwargs={'cluster': 'test_id', 'upgrade_type': 'cross'},
            expected_kwargs={
                'cluster_id': 'test_id',
                'upgrade_type': 'cross',
                'requires_id': False,
            },
        )

    def test_scale_in_cluster(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.scale_in',
            self.proxy.scale_in_cluster,
            method_args=[_cluster.Cluster, ['node-id']],
            expected_args=[self.proxy, ['node-id']],
        )

    def test_scale_in_cluster_by_node_type(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.scale_in_by_node_type',
            self.proxy.scale_in_cluster_by_node_type,
            method_args=[
                _cluster.Cluster,
                [{'type': 'node-type', 'reducedNodeNum': 1}],
            ],
            expected_args=[
                self.proxy,
                [{'type': 'node-type', 'reducedNodeNum': 1}],
            ],
        )

    def test_replace_cluster_node(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.replace_node',
            self.proxy.replace_cluster_node,
            method_args=[_cluster.Cluster, 'node-id'],
            expected_args=[self.proxy, 'node-id'],
        )

    def test_add_cluster_nodes(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.add_nodes',
            self.proxy.add_cluster_nodes,
            method_args=[
                _cluster.Cluster,
                'node-type',
                'flavor',
                3,
                'volume-type',
            ],
            expected_args=[
                self.proxy,
                'node-type',
                'flavor',
                3,
                'volume-type',
            ],
        )

    def test_get_cluster_upgrade_status(self):
        self.verify_list(
            self.proxy.get_cluster_upgrade_status,
            _cluster_upgrade_status.ClusterUpgradeStatus,
            method_kwargs={'cluster': 'cluster-uuid'},
            expected_kwargs={'cluster_id': 'cluster-uuid'},
        )

    def test_retry_cluster_upgrade_job(self):
        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.retry_upgrade_job',
            self.proxy.retry_cluster_upgrade_job,
            method_kwargs={
                'cluster': _cluster.Cluster,
                'job_id': 'job-id',
                'retry_mode': None,
            },
            expected_args=[self.proxy, 'job-id', None],
        )

        self._verify(
            'otcextensions.sdk.css.v1.cluster.Cluster.retry_upgrade_job',
            self.proxy.retry_cluster_upgrade_job,
            method_args=[_cluster.Cluster, 'job-id', 'abort'],
            expected_args=[self.proxy, 'job-id', 'abort'],
        )

    def test_flavors(self):
        self.verify_list(
            self.proxy.flavors,
            _flavor.Flavor,
        )

    def test_snapshots(self):
        self._verify(
            'openstack.proxy.Proxy._list',
            self.proxy.snapshots,
            method_args=['cluster-uuid'],
            expected_args=[_snapshot.Snapshot],
            expected_kwargs={
                'base_path': '/clusters/cluster-uuid/index_snapshots'
            },
        )

    def test_create_snapshot(self):
        self.verify_create(
            self.proxy.create_snapshot,
            _snapshot.Snapshot,
            method_kwargs={'cluster': 'cluster-uuid', 'x': 1, 'y': 2},
            expected_kwargs={
                'uri_cluster_id': 'cluster-uuid',
                'x': 1,
                'y': 2,
            },
        )

    def test_find_snapshot(self):
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_snapshot,
            method_args=['cluster-uuid', 'snapshot-uuid'],
            expected_args=[_snapshot.Snapshot, 'snapshot-uuid'],
            expected_kwargs={
                'base_path': '/clusters/cluster-uuid/index_snapshots',
                'ignore_missing': True,
            },
        )

    def test_delete_snapshot(self):
        self._verify(
            'openstack.proxy.Proxy._delete',
            self.proxy.delete_snapshot,
            method_args=['cluster-uuid', 'snapshot-uuid', False],
            expected_args=[_snapshot.Snapshot, 'snapshot-uuid'],
            expected_kwargs={
                'uri_cluster_id': 'cluster-uuid',
                'ignore_missing': False,
            },
        )

    def test_restore_snapshot(self):
        self._verify(
            'otcextensions.sdk.css.v1.snapshot.Snapshot.restore',
            self.proxy.restore_snapshot,
            method_args=[_cluster.Cluster, _snapshot.Snapshot],
            method_kwargs={'a': '1', 'b': '2'},
            expected_args=[self.proxy, _cluster.Cluster],
            expected_kwargs={'a': '1', 'b': '2'},
        )

    def test_set_snapshot_configuration(self):
        self.verify_create(
            self.proxy.set_snapshot_configuration,
            _snapshot.SnapshotConfiguration,
            method_kwargs={
                'cluster': 'cluster-uuid',
                'auto_configure': False,
                'x': 1,
                'y': 2,
            },
            expected_kwargs={
                'cluster_id': 'cluster-uuid',
                'setting': 'setting',
                'x': 1,
                'y': 2,
            },
        )

    def test_disable_snapshot_function(self):
        self._verify(
            'otcextensions.sdk.css.v1.snapshot.SnapshotConfiguration.disable',
            self.proxy.disable_snapshot_function,
            method_args=['cluster-uuid'],
            expected_args=[self.proxy],
        )

    def test_set_snapshot_policy(self):
        self.verify_create(
            self.proxy.set_snapshot_policy,
            _snapshot.SnapshotPolicy,
            method_kwargs={'cluster': 'cluster-uuid', 'x': 1, 'y': 2},
            expected_kwargs={'cluster_id': 'cluster-uuid', 'x': 1, 'y': 2},
        )

    def test_get_snapshot_policy(self):
        self._verify(
            'openstack.proxy.Proxy._get',
            self.proxy.get_snapshot_policy,
            method_args=['cluster-uuid'],
            expected_args=[_snapshot.SnapshotPolicy],
            expected_kwargs={
                'cluster_id': 'cluster-uuid',
                'requires_id': False,
            },
        )

    @mock.patch('builtins.open', new_callable=mock.mock_open)
    @mock.patch('otcextensions.sdk.css.v1._proxy.Proxy.get')
    def test_download_certificate(self, mock_get, mock_open):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {
            'Content-Disposition': 'attachment; filename="custom_cert.cer"'
        }
        mock_response.iter_content = mock.Mock(return_value=[b'cert content'])
        mock_get.return_value = mock_response

        filename = 'custom_cert.cer'
        self.proxy.download_certificate(filename)

        mock_get.assert_called_once_with(
            '/cer/download',
            headers={'Accept': '*/*'},
            stream=True
        )
        mock_open.assert_called_once_with(filename, 'wb')
        mock_open().write.assert_called_once_with(b'cert content')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.vpc.v2 import peering
from otcextensions.tests.unit.osclient.vpc.v1 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestListVpcPeerings(fakes.TestVpc):

    objects = fakes.FakeVpcPeering.create_multiple(3)

    column_list_headers = (
        'Id',
        'Name',
        'Status',
        'Local Router Id',
        'Peer Router Id',
        'Peer Project Id',
    )

    columns = (
        'id',
        'name',
        'status',
        'local_router_id',
        'peer_router_id',
        'peer_project_id'
    )

    data = []

    for s in objects:
        data.append(
            (s.id, s.name, s.status, s.local_vpc_info['vpc_id'],
                s.peer_vpc_info['vpc_id'], s.peer_vpc_info['tenant_id']))

    def setUp(self):
        super(TestListVpcPeerings, self).setUp()

        self.cmd = peering.ListVpcPeerings(self.app, None)

        self.client.peerings = mock.Mock()
        self.client.api_mock = self.client.peerings

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--limit', '1',
            '--marker', '2',
            '--id', '3',
            '--name', '4',
            '--project-id', '5',
            '--router-id', '6',
            '--status', 'ACTIVE'
        ]

        verifylist = [
            ('limit', 1),
            ('marker', '2'),
            ('id', '3'),
            ('name', '4'),
            ('project_id', '5'),
            ('router_id', '6'),
            ('status', 'ACTIVE'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            marker='2',
            id='3',
            name='4',
            project_id='5',
            router_id='6',
            status='ACTIVE',
        )


class TestCreateVpcPeering(fakes.TestVpc):

    _data = fakes.FakeVpcPeering.create_one()

    columns = (
        'id',
        'name',
        'local_vpc_info',
        'peer_vpc_info',
        'description',
        'created_at',
        'updated_at',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateVpcPeering, self).setUp()

        self.cmd = peering.CreateVpcPeering(self.app, None)
        self.client.create_peering = mock.Mock(
            return_value=fakes.FakeVpcPeering.create_one())
        self.client.get_project_id = mock.Mock(
            return_value='test-local-project-uuid')

    def test_create_different_project(self):
        arglist = [
            'test-peering',
            '--local-router-id', 'test-local-router-uuid',
            '--peer-router-id', 'test-peer-router-uuid',
            '--peer-project-id', 'test-peer-project-uuid',
            '--description', 'test-peering',
        ]
        verifylist = [
            ('name', 'test-peering'),
            ('local_router_id', 'test-local-router-uuid'),
            ('peer_router_id', 'test-peer-router-uuid'),
            ('peer_project_id', 'test-peer-project-uuid'),
            ('description', 'test-peering'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'name': 'test-peering',
            'request_vpc_info': {
                'vpc_id': 'test-local-router-uuid',
                'tenant_id': 'test-local-project-uuid'
            },
            'accept_vpc_info': {
                'vpc_id': 'test-peer-router-uuid',
                'tenant_id': 'test-peer-project-uuid'
            },
            'description': 'test-peering'
        }

        self.client.create_peering.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)

    def test_create_same_project(self):
        arglist = [
            'test-peering',
            '--local-router-id', 'test-local-router-uuid',
            '--peer-router-id', 'test-peer-router-uuid',
            '--description', 'test-peering',
        ]
        verifylist = [
            ('name', 'test-peering'),
            ('local_router_id', 'test-local-router-uuid'),
            ('peer_router_id', 'test-peer-router-uuid'),
            ('description', 'test-peering'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'name': 'test-peering',
            'request_vpc_info': {
                'vpc_id': 'test-local-router-uuid'
            },
            'accept_vpc_info': {
                'vpc_id': 'test-peer-router-uuid'
            },
            'description': 'test-peering'
        }

        self.client.create_peering.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)


class TestUpdateVpcPeering(fakes.TestVpc):

    _data = fakes.FakeVpcPeering.create_one()

    columns = (
        'id',
        'name',
        'local_vpc_info',
        'peer_vpc_info',
        'description',
        'created_at',
        'updated_at',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateVpcPeering, self).setUp()

        self.cmd = peering.UpdateVpcPeering(self.app, None)

        self.client.find_peering = mock.Mock(return_value=self._data)
        self.client.update_peering = mock.Mock(return_value=self._data)

    def test_update(self):
        arglist = [
            self._data.name,
            '--name', 'test-peering-updated',
            '--description', 'vpc peering updated',
        ]
        verifylist = [
            ('peering', self._data.name),
            ('name', 'test-peering-updated'),
            ('description', 'vpc peering updated'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_peering.assert_called_with(self._data.name)
        self.client.update_peering.assert_called_with(
            self._data.id,
            name='test-peering-updated',
            description='vpc peering updated'
        )
        self.assertEqual(self.columns, columns)


class TestShowVpcPeering(fakes.TestVpc):

    _data = fakes.FakeVpcPeering.create_one()

    columns = (
        'id',
        'name',
        'local_vpc_info',
        'peer_vpc_info',
        'description',
        'created_at',
        'updated_at',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowVpcPeering, self).setUp()

        self.cmd = peering.ShowVpcPeering(self.app, None)

        self.client.find_peering = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('peering', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_peering.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_vpc_peering',
        ]

        verifylist = [
            ('peering', 'unexist_vpc_peering'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_peering = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_peering.assert_called_with('unexist_vpc_peering')


class TestSetVpcPeering(fakes.TestVpc):

    _data = fakes.FakeVpcPeering.create_one()

    columns = (
        'id',
        'name',
        'local_vpc_info',
        'peer_vpc_info',
        'description',
        'created_at',
        'updated_at',
        'status'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestSetVpcPeering, self).setUp()

        self.cmd = peering.SetVpcPeering(self.app, None)

        self.client.find_peering = mock.Mock(return_value=self._data)
        self.client.set_peering = mock.Mock(return_value=self._data)

    def test_set(self):
        arglist = [
            self._data.name,
            '--accept'
        ]

        verifylist = [
            ('peering', self._data.name),
            ('accept', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_peering.assert_called_with(self._data.name)
        self.client.set_peering.assert_called_with(self._data.id, 'accept')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteVpcPeering(fakes.TestVpc):

    _data = fakes.FakeVpcPeering.create_multiple(2)

    def setUp(self):
        super(TestDeleteVpcPeering, self).setUp()

        self.client.delete_peering = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = peering.DeleteVpcPeering(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].name,
        ]

        verifylist = [
            ('peering', [self._data[0].name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.find_peering = (
            mock.Mock(return_value=self._data[0])
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_peering.assert_called_with(self._data[0].id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for data in self._data:
            arglist.append(data.name)

        verifylist = [
            ('peering', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = self._data
        self.client.find_peering = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for data in self._data:
            calls.append(call(data.id))
        self.client.delete_peering.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].name,
            'unexist_vpc_peering',
        ]
        verifylist = [
            ('peering', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [self._data[0], exceptions.CommandError]
        self.client.find_peering = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual(
                '1 of 2 VPC peering(s) failed to delete.', str(e))

        self.client.find_peering.assert_any_call(self._data[0].name)
        self.client.find_peering.assert_any_call('unexist_vpc_peering')
        self.client.delete_peering.assert_called_once_with(self._data[0].id)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create CCE Cluster node
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    'kind': 'Node',
    'apiVersion': 'v3',
    'metadata': {
        'name': 'myhost',
        'labels': {
            'foo': 'bar'
        },
        'annotations': {
            'annotation1': 'abc'
        }
    },
    'spec': {
        'flavor': 's2.large.2',
        'az': 'eu-de-02',
        'login': {
            'sshKey': 'keypair-pub'
        },
        'rootVolume': {
            'size': 40,
            'volumetype': 'SATA'
        },
        'dataVolumes': [
            {
                'size': 100,
                'volumetype': 'SATA'
            }
        ],
        'userTags': [
            {
                'key': 'tag1',
                'value': 'aaaa'
            },
            {
                'key': 'tag2',
                'value': 'bbbb'
            }
        ],
        'k8sTags': {
            'label-test': 'test'
        },
        'publicIP': {
            # ids: ['1234', '5678']
            'count': 2,
            'eip': {
                'iptype': '5_bgp',
                'bandwidth': {
                    'chargemode': 'traffic',
                    'size': 10,
                    'sharetype': 'PER'
                }
            }
        },
        'count': 2,
        'nodeNicSpec': {
            'primaryNic': {
                'subnetId': 'bbfc0a20-d66c-4f36-b4c1-265d669b8c62'
            }
        },
    }
}

cluster = 'name_or_id'
cluster = conn.cce.find_cluster(name_or_id=cluster)
conn.cce.create_cluster_node(cluster=cluster, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import metric_data

EXAMPLE_METRIC = {
    'average': 123,
    'max': 200,
    'min': 100,
    'sum': 1234,
    'variance': 12,
    'timestamp': '2025-01-01T12:00:00Z',
    'unit': 'ms'
}

EXAMPLE_RESPONSE = {
    'datapoints': [EXAMPLE_METRIC]
}


class TestMetricData(base.TestCase):

    def test_basic(self):
        sot = metric_data.MetricData()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/metric-data',
            sot.base_path
        )
        self.assertEqual('datapoints', sot.resources_key)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = metric_data.MetricData(**EXAMPLE_METRIC)
        self.assertEqual(123, sot.average)
        self.assertEqual(200, sot.max)
        self.assertEqual(100, sot.min)
        self.assertEqual(1234, sot.sum)
        self.assertEqual(12, sot.variance)
        self.assertEqual('2025-01-01T12:00:00Z', sot.timestamp)
        self.assertEqual('ms', sot.unit)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from otcextensions.osclient.rds.v3 import datastore
from otcextensions.tests.unit.osclient.rds.v3 import fakes


class TestListDatastores(fakes.TestRds):

    columns_datastore_type = ('Name', )

    def setUp(self):
        super(TestListDatastores, self).setUp()

        self.cmd = datastore.ListDatastoreTypes(self.app, None)

        self.app.client_manager.rds.datastore_types = mock.Mock()

        self.datastore_types = []
        self.datastore_type_data = []
        for ds in ['MySQL', 'PostgreSQL', 'SQLServer']:
            obj = type('obj', (object,), {'name': ds})
            self.datastore_types.append(obj)
            # Since only one field is there leave a comma at the end
            self.datastore_type_data.append((ds,))

    def test_list_datastore_types(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.datastore_types.side_effect = [
            self.datastore_types
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.datastore_types.assert_called()

        self.assertEqual(self.columns_datastore_type, columns)
        self.assertEqual(self.datastore_type_data, list(data))


class TestListDatastoreVersions(fakes.TestRds):
    objects = fakes.FakeDatastore.create_multiple(3)

    column_headers = ('ID', 'Name')
    columns = ('id', 'name')

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListDatastoreVersions, self).setUp()

        self.cmd = datastore.ListDatastoreVersions(self.app, None)

        self.client.datastores = mock.Mock(return_value=self.objects)

    def test_list_datastore_versions(self):
        arglist = [
            'MySQL'
        ]

        verifylist = [
            ('database', 'mysql')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.datastores.assert_called()

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.identity.v3 import _bad_base as _base


class CustomRole(_base.BadBaseResource):
    resources_key = 'roles'
    base_path = '/v3.0/OS-ROLE/roles'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'page', 'per_page'
    )

    #: Custom policy information.
    #: Properties
    #: ID of the domain which the custom policy belongs to.
    domain_id = resource.Body("domain_id")
    #: Number of references.
    references = resource.Body("references", type=int)
    #: Time when the custom policy was last updated.
    updated_at = resource.Body("updated_time")
    #: Time when the custom policy was created.
    created_at = resource.Body("created_time")
    #: Description of the custom policy.
    description_cn = resource.Body("description_cn")
    #: Service catalog.
    catalog = resource.Body("catalog")
    #: Name of the custom policy.
    name = resource.Body("name")
    #: Description of the custom policy.
    description = resource.Body("description")
    #: Resource link of the custom policy.
    links = resource.Body("links", type=dict)
    #: Policy ID.
    id = resource.Body("id")
    #: Display name of the custom policy.
    display_name = resource.Body("display_name")
    #: Display mode.
    type = resource.Body("type")
    #: Content of custom policy.
    policy = resource.Body("policy", type=dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""SMN template v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


PROTOCOL_CHOICES = ['email', 'default', 'sms', 'dms', 'http', 'https']


class ListTemplate(command.Lister):

    _description = _("List message templates.")
    columns = ('ID', 'Name', 'Protocol')

    def get_parser(self, prog_name):
        parser = super(ListTemplate, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Message template name."),
        )
        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(PROTOCOL_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=PROTOCOL_CHOICES,
            help=_("Protocol supported by the template."),
        )
        parser.add_argument(
            '--offset',
            metavar='<offset>',
            type=int,
            help=_("Resources after this offset will be queried."),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Limit to fetch number of records."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn
        args_list = [
            'limit',
            'offset',
            'protocol',
            'name']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.templates(**attrs)

        return (self.columns, (utils.get_item_properties(s, self.columns)
                               for s in data))


class ShowTemplate(command.ShowOne):
    _description = _("Show message template details.")

    def get_parser(self, prog_name):
        parser = super(ShowTemplate, self).get_parser(prog_name)
        parser.add_argument(
            'template',
            metavar='<template>',
            help=_("Specifies the Name or ID of the message template."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn
        obj = client.find_template(parsed_args.template)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class CreateTemplate(command.ShowOne):
    _description = _("Create new message template.")

    def get_parser(self, prog_name):
        parser = super(CreateTemplate, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Specifies the name of the message template."),
        )
        parser.add_argument(
            '--content',
            metavar='<content>',
            required=True,
            help=_("Template content, which currently supports "
                   "plain text only."),
        )
        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(PROTOCOL_CHOICES) + '}',
            type=lambda s: s.lower(),
            choices=PROTOCOL_CHOICES,
            help=_("Protocol supported by the template."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn

        args_list = [
            'name',
            'protocol',
            'content']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.create_template(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UpdateTemplate(command.ShowOne):
    _description = _("Update a message template.")

    def get_parser(self, prog_name):
        parser = super(UpdateTemplate, self).get_parser(prog_name)
        parser.add_argument(
            'template',
            metavar='<template>',
            help=_("Specifies the Name or ID of the message template."),
        )
        parser.add_argument(
            '--content',
            metavar='<content>',
            required=True,
            help=_("Template content, which currently supports "
                   "plain text only."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn
        attrs = {
            'content': parsed_args.content
        }
        template = client.find_template(parsed_args.template)

        obj = client.update_template(template, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteTemplate(command.Command):

    _description = _("Deletes message template.")

    def get_parser(self, prog_name):
        parser = super(DeleteTemplate, self).get_parser(prog_name)
        parser.add_argument(
            'template',
            metavar='<template>',
            nargs='+',
            help=_("message template(s) to delete (Name or ID)"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn
        result = 0
        for template in parsed_args.template:
            try:
                obj = client.find_template(template)
                client.delete_template(obj)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete message template with "
                          "name or ID '%(template)s': %(e)s"),
                          {'template': template, 'e': e})
        if result > 0:
            total = len(parsed_args.template)
            msg = (_("%(result)s of %(total)s message template(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import resource

from otcextensions.sdk import sdk_resource

_logger = _log.setup_logging('openstack')


class Datastore(sdk_resource.Resource):

    base_path = '/datastores/%(datastore_name)s/versions'
    resource_key = ''
    resources_key = 'dataStores'

    # capabilities
    allow_get = False
    allow_list = True

    project_id = resource.URI('project_id')
    datastore_name = resource.URI('datastore_name')

    # Indicates the database version ID. Its value is unique.
    # :*Type:string*
    id = resource.Body('id')
    # Indicates the database version.
    # :*Type:string*
    name = resource.Body('name')
    # Indicates the database ID.
    #: *Type:string*
    datastore = resource.Body('datastore')
    # Indicates the database image ID.
    # :*Type:string*
    image = resource.Body('image')
    #: Indicates the database package version information.
    #: *Type:string*
    packages = resource.Body('packages')
    # Indicates the current database version status. 0 indicates
    # Non-activated, and 1 indicates Activated.
    # The interface can only query information of versions that are activated.
    #: *Type:int*
    active = resource.Body('active', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.sfsturbo.v1 import share as _share


IDENTIFIER = 'ID'
EXAMPLE = {
    "id": "share-id",
    "action_progress": {},
    "az_name": "az_name",
    "avail_capacity": "avail_capacity",
    "availability_zone": "availability_zone",
    "created_at": "created_at",
    "crypt_key_id": "crypt_key_id",
    "expand_type": "expand_type",
    "export_location": "export_location",
    "status": "status",
    "sub_status": "sub_status",
    "share_type": "share_type",
    "subnet_id": "subnet_id",
    "security_group_id": "security_group_id",
    "size": "size",
    "share_proto": "share_proto",
    "pay_model": "pay_model",
    "version": "version",
    "vpc_id": "vpc_id"
}


class TestShare(base.TestCase):

    def setUp(self):
        super(TestShare, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = _share.Share()
        self.assertEqual('share', sot.resource_key)
        self.assertEqual('shares', sot.resources_key)
        path = '/sfs-turbo/shares'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = _share.Share(**EXAMPLE)
        self.assertEqual(EXAMPLE['az_name'], sot.az_name)
        self.assertEqual(EXAMPLE['action_progress'], sot.action_progress),
        self.assertEqual(EXAMPLE['avail_capacity'], sot.avail_capacity)
        self.assertEqual(EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['crypt_key_id'], sot.crypt_key_id)
        self.assertEqual(EXAMPLE['expand_type'], sot.expand_type)
        self.assertEqual(EXAMPLE['export_location'], sot.export_location)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['sub_status'], sot.sub_status)
        self.assertEqual(EXAMPLE['share_type'], sot.share_type)
        self.assertEqual(EXAMPLE['subnet_id'], sot.subnet_id)
        self.assertEqual(EXAMPLE['security_group_id'], sot.security_group_id)
        self.assertEqual(EXAMPLE['size'], sot.size)
        self.assertEqual(EXAMPLE['share_proto'], sot.share_proto)
        self.assertEqual(EXAMPLE['pay_model'], sot.pay_model)
        self.assertEqual(EXAMPLE['version'], sot.version)
        self.assertEqual(EXAMPLE['vpc_id'], sot.vpc_id)

    def test_expand_capacity(self):
        mock_response = mock.Mock()
        mock_response.status_code = 202
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response
        self.sess.default_microversion = '1'

        sot = _share.Share.existing(id=EXAMPLE['id'])

        sot.extend_capacity(self.sess, {'new_size': 5})

        self.sess.post.assert_called_once_with(
            'sfs-turbo/shares/%s/action' % EXAMPLE['id'],
            json={'extend': {'new_size': 5}}
        )

    def test_change_security_group(self):
        mock_response = mock.Mock()
        mock_response.status_code = 202
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response
        self.sess.default_microversion = '1'

        sot = _share.Share.existing(id=EXAMPLE['id'])

        sot.change_security_group(self.sess,
                                  {'security_group_id': 'secgroup-uuid'})

        self.sess.post.assert_called_once_with(
            'sfs-turbo/shares/%s/action' % EXAMPLE['id'],
            json={'change_security_group': {
                'security_group_id': 'secgroup-uuid'}}
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.osclient.auto_scaling.v1 import group
from otcextensions.tests.unit.osclient.auto_scaling.v1 import fakes


class TestAutoScalingGroup(fakes.TestAutoScaling):

    def setUp(self):
        super(TestAutoScalingGroup, self).setUp()
        self.client = self.app.client_manager.auto_scaling


class TestListAutoScalingGroup(TestAutoScalingGroup):

    groups = fakes.FakeGroup.create_multiple(3)

    columns = ('ID', 'Name', 'status', 'detail')

    data = []

    for s in groups:
        data.append((
            s.id,
            s.name,
            s.status,
            s.detail
        ))

    def setUp(self):
        super(TestListAutoScalingGroup, self).setUp()

        self.cmd = group.ListAutoScalingGroup(self.app, None)

        self.client.groups = mock.Mock()

    def test_list_default(self):
        arglist = [
            '--name', 'grp',
            '--status', 'PAUSED',
            '--scaling-configuration-id', '2',
            '--limit', '12'
        ]

        verifylist = [
            ('name', 'grp'),
            ('status', 'PAUSED'),
            ('scaling_configuration_id', '2'),
            ('limit', 12)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.groups.side_effect = [
            self.groups
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.groups.assert_called_once_with(
            name='grp',
            status='PAUSED',
            scaling_configuration_id='2',
            limit=12)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowAutoScalingGroup(TestAutoScalingGroup):

    columns = ('create_time', 'detail', 'id', 'name', 'router_id', 'status')

    _group = fakes.FakeGroup.create_one()

    data = (
        _group.create_time,
        _group.detail,
        _group.id,
        _group.name,
        _group.router_id,
        _group.status,
    )

    def setUp(self):
        super(TestShowAutoScalingGroup, self).setUp()

        self.cmd = group.ShowAutoScalingGroup(self.app, None)

        self.client.find_group = mock.Mock()

    def test_show_default(self):
        arglist = [
            'test_group'
        ]
        verifylist = [
            ('group', 'test_group')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_group.side_effect = [
            self._group
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_group.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateAutoScalingGroup(TestAutoScalingGroup):

    columns = ('create_time', 'detail', 'id', 'name', 'router_id', 'status')

    _group = fakes.FakeGroup.create_one()

    data = (
        _group.create_time,
        _group.detail,
        _group.id,
        _group.name,
        _group.router_id,
        _group.status,
    )

    def setUp(self):
        super(TestCreateAutoScalingGroup, self).setUp()

        self.cmd = group.CreateAutoScalingGroup(self.app, None)

        self.client.create_group = mock.Mock()

    def test_create(self):
        arglist = [
            '--desire-instance-number', '10',
            '--min-instance-number', '1',
            '--max-instance-number', '15',
            '--cool-down-time', '1',
            '--availability-zone', 'eu-1',
            '--availability-zone', 'eu-2',
            '--network-id', 'sub1',
            '--network-id', 'sub2',
            '--router-id', 'vpc-1',
            '--security-group', 'sg1',
            '--security-group', 'sg2',
            '--lb-listener-id', 'lb1',
            '--lbaas-listener', 'lbas1:14',
            '--lbaas-listener', 'lbas2:15:10',
            '--audit-method', 'some_method',
            '--audit-time', '15',
            '--terminate-policy', 'pol',
            '--notification', 'EMAIL',
            '--notification', 'SMS',

            'test_name'
        ]
        verifylist = [
            ('desire_instance_number', 10),
            ('min_instance_number', 1),
            ('max_instance_number', 15),
            ('cool_down_time', 1),
            ('availability_zone', ['eu-1', 'eu-2']),
            ('network_id', ['sub1', 'sub2']),
            ('security_group', ['sg1', 'sg2']),
            ('router_id', 'vpc-1'),
            ('lb_listener_id', 'lb1'),
            ('lbaas_listener', ['lbas1:14', 'lbas2:15:10']),
            ('audit_method', 'some_method'),
            ('audit_time', 15),
            ('terminate_policy', 'pol'),
            ('notification', ['EMAIL', 'SMS']),
            ('name', 'test_name')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_group.side_effect = [
            self._group
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_group.assert_called_with(
            available_zones=['eu-1', 'eu-2'],
            cool_down_time=1,
            desire_instance_number=10,
            health_periodic_audit_method='some_method',
            health_periodic_audit_time=15,
            instance_terminate_policy='pol',
            lb_listener_id='lb1',
            lbaas_listeners=[
                {'id': 'lbas1', 'protocol_port': '14'},
                {'id': 'lbas2', 'protocol_port': '15', 'weight': '10'}],
            max_instance_number=15,
            min_instance_number=1,
            name='test_name',
            networks=[{'id': 'sub1'}, {'id': 'sub2'}],
            notifications=['EMAIL', 'SMS'],
            security_groups=[{'id': 'sg1'}, {'id': 'sg2'}],
            router_id='vpc-1'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteAutoScalingGroup(TestAutoScalingGroup):

    def setUp(self):
        super(TestDeleteAutoScalingGroup, self).setUp()

        self.cmd = group.DeleteAutoScalingGroup(self.app, None)

        self.client.delete_group = mock.Mock()

    def test_sdelete(self):
        arglist = [
            'group1'
        ]
        verifylist = [
            ('group', 'group1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_group.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_group.assert_called()


class TestUpdateAutoScalingGroup(TestAutoScalingGroup):

    columns = ('create_time', 'detail', 'id', 'name', 'router_id', 'status')

    _group = fakes.FakeGroup.create_one()

    data = (
        _group.create_time,
        _group.detail,
        _group.id,
        _group.name,
        _group.router_id,
        _group.status,
    )

    def setUp(self):
        super(TestUpdateAutoScalingGroup, self).setUp()

        self.cmd = group.UpdateAutoScalingGroup(self.app, None)

        self.client.update_group = mock.Mock()

    def test_create(self):
        arglist = [
            '--desire-instance-number', '10',
            '--min-instance-number', '1',
            '--max-instance-number', '15',
            '--cool-down-time', '1',
            '--availability-zone', 'eu-1',
            '--availability-zone', 'eu-2',
            '--network-id', 'sub1',
            '--network-id', 'sub2',
            '--router-id', 'vpc-1',
            '--security-group', 'sg1',
            '--security-group', 'sg2',
            '--lb-listener-id', 'lb1',
            '--lbaas-listener', 'lbas1:14',
            '--lbaas-listener', 'lbas2:15:10',
            '--audit-method', 'some_method',
            '--audit-time', '15',
            '--terminate-policy', 'pol',
            '--notification', 'EMAIL',
            '--notification', 'SMS',

            'test_name'
        ]
        verifylist = [
            ('desire_instance_number', 10),
            ('min_instance_number', 1),
            ('max_instance_number', 15),
            ('cool_down_time', 1),
            ('availability_zone', ['eu-1', 'eu-2']),
            ('network_id', ['sub1', 'sub2']),
            ('security_group', ['sg1', 'sg2']),
            ('router_id', 'vpc-1'),
            ('lb_listener_id', 'lb1'),
            ('lbaas_listener', ['lbas1:14', 'lbas2:15:10']),
            ('audit_method', 'some_method'),
            ('audit_time', 15),
            ('terminate_policy', 'pol'),
            ('notification', ['EMAIL', 'SMS']),
            ('group', 'test_name')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_group.side_effect = [
            self._group
        ]

        self.client.find_group.side_effect = [
            self._group
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_group.assert_called_with(
            self._group,
            available_zones=['eu-1', 'eu-2'],
            cool_down_time=1,
            desire_instance_number=10,
            health_periodic_audit_method='some_method',
            health_periodic_audit_time=15,
            instance_terminate_policy='pol',
            lb_listener_id='lb1',
            lbaas_listeners=[
                {'id': 'lbas1', 'protocol_port': '14'},
                {'id': 'lbas2', 'protocol_port': '15', 'weight': '10'}],
            max_instance_number=15,
            min_instance_number=1,
            networks=[{'id': 'sub1'}, {'id': 'sub2'}],
            notifications=['EMAIL', 'SMS'],
            security_groups=[{'id': 'sg1'}, {'id': 'sg2'}],
            router_id='vpc-1'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestEnableAutoScalingGroup(TestAutoScalingGroup):

    _group = fakes.FakeGroup.create_one()

    def setUp(self):
        super(TestEnableAutoScalingGroup, self).setUp()

        self.cmd = group.EnableAutoScalingGroup(self.app, None)

        self.client.find_group = mock.Mock()
        self.client.resume_group = mock.Mock()

    def test_enable(self):
        arglist = [
            'group1'
        ]
        verifylist = [
            ('group', 'group1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_group.side_effect = [
            self._group
        ]
        self.client.resume_group.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_group.assert_called()
        self.client.resume_group.assert_called()


class TestDisableAutoScalingGroup(TestAutoScalingGroup):

    _group = fakes.FakeGroup.create_one()

    def setUp(self):
        super(TestDisableAutoScalingGroup, self).setUp()

        self.cmd = group.DisableAutoScalingGroup(self.app, None)

        self.client.find_group = mock.Mock()
        self.client.pause_group = mock.Mock()

    def test_disable(self):
        arglist = [
            'group1'
        ]
        verifylist = [
            ('group', 'group1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_group.side_effect = [
            self._group
        ]
        self.client.pause_group.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_group.assert_called()
        self.client.pause_group.assert_called()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv2.v2 import dataset_version
from otcextensions.tests.unit.sdk.modelartsv2.v2 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.DATASET_VERSION

EXAMPLE_CREATE = {
    "version_name": "V004",
    "version_format": "Default",
    "description": "",
    "clear_hard_property": True,
}


class TestDataset(base.TestCase):
    def setUp(self):
        super(TestDataset, self).setUp()

    def test_basic(self):
        sot = dataset_version.DatasetVersion()

        self.assertEqual("/datasets/%(dataset_id)s/versions", sot.base_path)
        self.assertEqual("versions", sot.resources_key)
        self.assertEqual(None, sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

        self.assertDictEqual(
            {
                "limit": "limit",
                "marker": "marker",
                "status": "status",
                "train_evaluate_ratio": "train_evaluate_ratio",
                "version_format": "version_format",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = dataset_version.DatasetVersion(**EXAMPLE)
        self.assertEqual(sot.id, EXAMPLE["version_id"])
        for key, value in EXAMPLE.items():
            if key == "create_time":
                self.assertEqual(sot.created_at, EXAMPLE[key])
            elif key == "label_stats":
                for ix, data in enumerate(value):
                    self.assertEqual(sot.label_stats[ix].name, data["name"])
                    self.assertEqual(sot.label_stats[ix].type, data["type"])
                    self.assertEqual(sot.label_stats[ix].count, data["count"])
                    self.assertEqual(
                        sot.label_stats[ix].sample_count, data["sample_count"]
                    )
                    self.assertEqual(
                        sot.label_stats[ix].attributes, data["attributes"]
                    )
                    self.assertEqual(
                        sot.label_stats[ix].property.color,
                        data["property"]["@modelarts:color"],
                    )
                    self.assertEqual(
                        sot.label_stats[ix].property.shortcut,
                        data["property"]["@modelarts:shortcut"],
                    )
            else:
                assert_attributes_equal(self, getattr(sot, key), value)

    def test_create_sot(self):
        sot = dataset_version.DatasetVersion(**EXAMPLE_CREATE)
        assert_attributes_equal(self, sot, EXAMPLE_CREATE)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.dcaas.v2 import connection
from otcextensions.tests.unit.osclient.dcaas.v2 import fakes

from osc_lib import exceptions

from openstackclient.tests.unit import utils as tests_utils


class TestListDirectConnection(fakes.TestDcaas):

    objects = fakes.FakeDirectConnection.create_multiple(3)
    column_list_headers = (
        'id',
        'name',
        'port type',
        'provider',
        'bandwidth',
        'location',
        'status'
    )
    columns = (
        'id',
        'name',
        'port type',
        'provider',
        'bandwidth',
        'location',
        'status'
    )
    data = []
    for s in objects:
        data.append((
            s.id,
            s.name,
            s.port_type,
            s.provider,
            s.bandwidth,
            s.location,
            s.status
        ))

    def setUp(self):
        super(TestListDirectConnection, self).setUp()
        self.cmd = connection.ListDirectConnections(self.app, None)
        self.client.connections = mock.Mock()
        self.client.api_mock = self.client.connections

    def test_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.api_mock.side_effect = [self.objects]
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()
        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--id', '1',
            '--name', 'test',
            '--port_type', '1G',
            '--bandwidth', '10',
            '--location', 'Biere',
            '--provider', 'OTC'
        ]
        verifylist = [
            ('id', '1'),
            ('name', 'test'),
            ('port_type', '1G'),
            ('bandwidth', 10),
            ('location', 'Biere'),
            ('provider', 'OTC')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.api_mock.side_effect = [self.objects]
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            id='1',
            name='test',
            port_type='1G',
            bandwidth=10,
            location='Biere',
            provider='OTC'
        )


class TestCreateDirectConnection(fakes.TestDcaas):

    _data = fakes.FakeDirectConnection.create_one()

    columns = (
        'admin_state_up',
        'bandwidth',
        'charge_mode',
        'description',
        'device_id',
        'hosting_id',
        'id',
        'interface_name',
        'location',
        'name',
        'order_id',
        'peer_location',
        'port_type',
        'product_id',
        'provider',
        'provider_status',
        'redundant_id',
        'status',
        'type',
        'vlan'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateDirectConnection, self).setUp()
        self.cmd = connection.CreateDirectConnection(self.app, None)
        self.client.create_connection = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            '1G',
            '10',
            'Biere',
            'OTC',
            '--name', 'test',
            '--description', 'test description',
            '--peer_location', 'test_peer_loc',
            '--device_id', '172.16.40.2',
            '--interface_name', 'Eth-Trunk2',
            '--redundant_id', '11111',
            '--provider_status', 'ACTIVE',
            '--type', 'hosted',
            '--hosting_id', 'test_11',
            '--vlan', '111',
            '--charge_mode', 'traffic',
            '--order_id', 'id1',
            '--product_id', 'idp1',
            '--status', 'ACTIVE',
            '--admin_state_up', 'True'
        ]
        verifylist = [
            ('port_type', '1G'),
            ('bandwidth', 10),
            ('location', 'Biere'),
            ('provider', 'OTC'),
            ('name', 'test'),
            ('description', 'test description'),
            ('peer_location', 'test_peer_loc'),
            ('device_id', '172.16.40.2'),
            ('interface_name', 'Eth-Trunk2'),
            ('redundant_id', '11111'),
            ('provider_status', 'ACTIVE'),
            ('type', 'hosted'),
            ('hosting_id', 'test_11'),
            ('vlan', 111),
            ('charge_mode', 'traffic'),
            ('order_id', 'id1'),
            ('product_id', 'idp1'),
            ('status', 'ACTIVE'),
            ('admin_state_up', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.client.create_connection.assert_called_with(
            name='test',
            port_type='1G',
            bandwidth=10,
            location='Biere',
            provider='OTC',
            description='test description',
            peer_location='test_peer_loc',
            device_id='172.16.40.2',
            interface_name='Eth-Trunk2',
            redundant_id='11111',
            provider_status='ACTIVE',
            type='hosted',
            hosting_id='test_11',
            vlan=111,
            charge_mode='traffic',
            order_id='id1',
            product_id='idp1',
            status='ACTIVE',
            admin_state_up=True
        )
        self.assertEqual(self.columns, columns)


class TestShowDirectConnection(fakes.TestDcaas):

    _data = fakes.FakeDirectConnection.create_one()

    columns = (
        'admin_state_up',
        'bandwidth',
        'charge_mode',
        'description',
        'device_id',
        'hosting_id',
        'id',
        'interface_name',
        'location',
        'name',
        'order_id',
        'peer_location',
        'port_type',
        'product_id',
        'provider',
        'provider_status',
        'redundant_id',
        'status',
        'type',
        'vlan'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowDirectConnection, self).setUp()
        self.cmd = connection.ShowDirectConnection(self.app, None)
        self.client.find_connection = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('direct_connection', self._data.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_connection.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existing(self):
        arglist = [
            'unexist_direct_connection'
        ]

        verifylist = [
            ('direct_connection', 'unexist_direct_connection')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_connection = (
            mock.Mock(side_effect=find_mock_result)
        )
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_connection.assert_called_with(
            'unexist_direct_connection'
        )


class TestUpdateDirectConnection(fakes.TestDcaas):

    _data = fakes.FakeDirectConnection.create_one()

    columns = (
        'admin_state_up',
        'bandwidth',
        'charge_mode',
        'description',
        'device_id',
        'hosting_id',
        'id',
        'interface_name',
        'location',
        'name',
        'order_id',
        'peer_location',
        'port_type',
        'product_id',
        'provider',
        'provider_status',
        'redundant_id',
        'status',
        'type',
        'vlan'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateDirectConnection, self).setUp()
        self.cmd = connection.UpdateDirectConnection(self.app, None)
        self.client.find_connection = mock.Mock(return_value=self._data)
        self.client.update_connection = mock.Mock(return_value=self._data)
        self.client.api_mock = self.client.update_connection

    def test_update(self):
        arglist = [
            self._data.id,
            '--name', 'updated_name',
            '--description', 'updated description',
            '--bandwidth', '150',
            '--provider_status', 'DOWN',
            '--order_id', 'id2',
            '--product_id', 'idp2'
        ]
        verifylist = [
            ('direct_connection', self._data.id),
            ('name', 'updated_name'),
            ('description', 'updated description'),
            ('bandwidth', 150),
            ('provider_status', 'DOWN'),
            ('order_id', 'id2'),
            ('product_id', 'idp2')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.side_effect = [
            self._data
        ]

        self.client.api_mock.assert_called_with(
            self._data.id,
            name='updated_name',
            description='updated description',
            bandwidth=150,
            provider_status='DOWN',
            order_id='id2',
            product_id='idp2'
        )
        self.assertEqual(self.columns, columns)


class TestDeleteDirectConnection(fakes.TestDcaas):
    _data = fakes.FakeDirectConnection.create_one()

    def setUp(self):
        super(TestDeleteDirectConnection, self).setUp()
        self.client.delete_connection = mock.Mock(return_value=None)
        self.cmd = connection.DeleteDirectConnection(self.app, None)

    def test_delete_by_name(self):
        arglist = [
            self._data.name,
        ]
        verifylist = [
            ('direct_connection', self._data.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.client.find_connection = (
            mock.Mock(return_value=self._data)
        )
        result = self.cmd.take_action(parsed_args)
        self.client.delete_connection.assert_called_with(self._data.id)
        self.assertIsNone(result)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from openstackclient.tests.unit import utils

from otcextensions.common import sdk_utils
from otcextensions.osclient.load_balancer.v1 import health_monitor
from otcextensions.tests.unit.osclient.load_balancer.v1 import fakes


class TestListHealthMonitor(fakes.TestLoadBalancer):

    _objects = fakes.FakeHealthMonitor.create_multiple(3)

    columns = ('id', 'name', 'project_id', 'type', 'admin_state_up')

    data = []

    for s in _objects:
        data.append((
            s.id,
            s.name,
            s.project_id,
            s.type,
            s.is_admin_state_up,
        ))

    def setUp(self):
        super(TestListHealthMonitor, self).setUp()

        self.cmd = health_monitor.ListHealthMonitor(self.app, None)

        self.client.health_monitors = mock.Mock()

    def test_list_default(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.health_monitors.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.health_monitors.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))

    def test_list_filter_values(self):
        arglist = [
            # '--delay', '1',
            # '--expected_codes', 'codes',
            # '--http_method', 'GET',
            # '--max_retries', '2',
            # '--timeout', '3',
            # '--url_path', 'url_path',
            '--type', 'HTTP',
        ]

        verifylist = [
            # ('delay', 1),
            # ('max_retries', 2),
            # ('timeout', 3),
            # ('expected_codes', 'codes'),
            # ('http_method', 'GET'),
            # ('url_path', 'url_path'),
            ('type', 'HTTP'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.health_monitors.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.health_monitors.assert_called_once_with(
            # delay=1,
            # max_retries=2,
            # timeout=3,
            # expected_codes='codes',
            # http_method='GET',
            # url_path='url_path',
            type='HTTP'
        )

    def test_list_filter_exceptions_method(self):
        arglist = [
            '--http_method', 'bad',
        ]

        verifylist = [
            ('http_method', 'bad')
        ]

        # Ensure exception is raised
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_list_filter_exceptions_type(self):
        arglist = [
            '--type', 'bad',
        ]

        verifylist = [
            ('type', 'bad')
        ]

        # Ensure exception is raised
        self.assertRaises(
            utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)


class TestShowHealthMonitor(fakes.TestLoadBalancer):

    _object = fakes.FakeHealthMonitor.create_one()

    columns = (
        'admin_state_up', 'expected_codes', 'http_method',
        'id', 'max_retries', 'name', 'pool_ids', 'timeout',
        'type', 'url_path')

    data = (
        _object.is_admin_state_up,
        _object.expected_codes,
        _object.http_method,
        _object.id,
        _object.max_retries,
        _object.name,
        sdk_utils.ListOfIdsColumnBR(_object.pool_ids),
        _object.timeout,
        _object.type,
        _object.url_path,
    )

    def setUp(self):
        super(TestShowHealthMonitor, self).setUp()

        self.cmd = health_monitor.ShowHealthMonitor(self.app, None)

        self.client.find_health_monitor = mock.Mock()

    def test_show_default(self):
        arglist = [
            'hm'
        ]

        verifylist = [
            ('health_monitor', 'hm'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_health_monitor.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_health_monitor.assert_called_once_with(
            name_or_id='hm',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestCreateHealthMonitor(fakes.TestLoadBalancer):

    _object = fakes.FakeHealthMonitor.create_one()
    _pool = fakes.FakePool.create_one()

    columns = (
        'admin_state_up', 'expected_codes', 'http_method',
        'id', 'max_retries', 'name', 'pool_ids', 'timeout',
        'type', 'url_path')

    data = (
        _object.is_admin_state_up,
        _object.expected_codes,
        _object.http_method,
        _object.id,
        _object.max_retries,
        _object.name,
        sdk_utils.ListOfIdsColumnBR(_object.pool_ids),
        _object.timeout,
        _object.type,
        _object.url_path,
    )

    def setUp(self):
        super(TestCreateHealthMonitor, self).setUp()

        self.cmd = health_monitor.CreateHealthMonitor(self.app, None)

        self.client.create_health_monitor = mock.Mock()
        self.client.find_pool = mock.Mock()

    def test_create_default(self):
        arglist = [
            'pool_id',
            '--disable',
            '--delay', '1',
            '--expected_codes', '100',
            '--http_method', 'CONNECT',
            '--name', 'name',
            '--max_retries', '2',
            '--timeout', '3',
            '--type', 'PING',
            '--url_path', 'url'
        ]

        verifylist = [
            ('disable', True),
            ('delay', 1),
            ('expected_codes', '100'),
            ('http_method', 'CONNECT'),
            ('name', 'name'),
            ('max_retries', 2),
            ('pool', 'pool_id'),
            ('timeout', 3),
            ('type', 'PING'),
            ('url_path', 'url')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_health_monitor.side_effect = [
            self._object
        ]
        self.client.find_pool.side_effect = [
            self._pool
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_health_monitor.assert_called_once_with(
            is_admin_state_up=False,
            delay=1,
            expected_codes='100',
            http_method='CONNECT',
            max_retries=2,
            name='name',
            pool_id=self._pool.id,
            timeout=3,
            type='PING',
            url_path='url'
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestUpdateHealthMonitor(fakes.TestLoadBalancer):

    _object = fakes.FakeHealthMonitor.create_one()

    columns = (
        'admin_state_up', 'expected_codes', 'http_method',
        'id', 'max_retries', 'name', 'pool_ids', 'timeout',
        'type', 'url_path')

    data = (
        _object.is_admin_state_up,
        _object.expected_codes,
        _object.http_method,
        _object.id,
        _object.max_retries,
        _object.name,
        sdk_utils.ListOfIdsColumnBR(_object.pool_ids),
        _object.timeout,
        _object.type,
        _object.url_path,
    )

    def setUp(self):
        super(TestUpdateHealthMonitor, self).setUp()

        self.cmd = health_monitor.SetHealthMonitor(self.app, None)

        self.client.update_health_monitor = mock.Mock()
        self.client.find_health_monitor = mock.Mock()

    def test_update_default(self):
        arglist = [
            'hm',
            '--disable',
            '--delay', '1',
            '--expected_codes', '100',
            '--http_method', 'CONNECT',
            '--name', 'name',
            '--max_retries', '2',
            '--timeout', '3',
            '--url_path', 'url'
        ]

        verifylist = [
            ('health_monitor', 'hm'),
            ('disable', True),
            ('delay', 1),
            ('expected_codes', '100'),
            ('http_method', 'CONNECT'),
            ('name', 'name'),
            ('max_retries', 2),
            ('timeout', 3),
            ('url_path', 'url')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_health_monitor.side_effect = [
            self._object
        ]
        self.client.find_health_monitor.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_health_monitor.assert_called_once_with(
            health_monitor=self._object.id,
            is_admin_state_up=False,
            delay=1,
            expected_codes='100',
            http_method='CONNECT',
            max_retries=2,
            name='name',
            timeout=3,
            url_path='url'
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestDeleteHealthMonitor(fakes.TestLoadBalancer):

    _object = fakes.FakeHealthMonitor.create_one()

    def setUp(self):
        super(TestDeleteHealthMonitor, self).setUp()

        self.cmd = health_monitor.DeleteHealthMonitor(self.app, None)

        self.client.delete_health_monitor = mock.Mock()
        self.client.find_health_monitor = mock.Mock()

    def test_delete_default(self):
        arglist = [
            'hm',
        ]

        verifylist = [
            ('health_monitor', ['hm']),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_health_monitor.side_effect = [
            {}
        ]
        self.client.find_health_monitor.side_effect = [
            self._object
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_health_monitor.assert_called_once_with(
            health_monitor=self._object.id,
            ignore_missing=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class OperationDefinition(resource.Resource):
    #: Properties
    #: Specifies the number of retained daily backups
    #: The latest backup of each day is saved in the long term. This parameter
    #: can be effective together with the max number of retained backups
    #: specified by 'max_backups'.0
    #: value: 0 to 100
    #: If this property is configured, 'timezone' is mandatory.
    day_backups = resource.Body('day_backups', type=int)
    #: Maximum number of retained backups.
    #: value: -1 or ranges from 0 to 99999
    #: If -1 is set, the backups will not be cleared even though the
    #: configured retained backup quantity is exceeded.
    #: If this parameter and 'retention_duration_days' are both left blank,
    #: the backups will be retained permanently
    max_backups = resource.Body('max_backups', type=int)
    #: Specifies the number of retained monthly backups.
    #: The latest backup of each month is saved in the long term. This
    #: parameter can be effective together with the max number of  retained
    #: backups specified by 'max_backups'.
    #: value: ranges from 0 to 100
    #: If this parameter is configured 'timezone' is mandatory.
    month_backups = resource.Body('month_backups', type=int)
    #: Duration of retaining a backup in days
    #: value: -1 or ranges from 0 to 99999
    #: -1 indicates that the backups will not be cleared bassed on the
    #: retention duration. if this parameter and 'max_backups' are left blank
    #: at the same time, the backups will be retained permanently.
    retention_duration_days = resource.Body('retention_duration_days',
                                            type=int)
    #: Time where the user is located, e.g.: UTC+08:00
    #: This paramter is only configurable if 'day_backups, 'week_backups',
    #: 'month_backups' or 'year_backups' are set.
    timezone = resource.Body('timezone')
    #: Specifies the number or retained weekly backups.
    #: The latest backup of each week is saved in the long term. This
    #: param can be effective together with the max number of retained
    #: backups specified by 'max_backups'. If this param is configured,
    #: 'timezone' is mandatory.
    #: value: ranges from 0 to 100
    week_backups = resource.Body('week_backups', type=int)
    #: Specifies the number or retained yearly backups.
    #: The latest backup of each year is saved in the long term. This
    #: param can be effective together with the max number of retained
    #: backups specified by 'max_backups'. If this param is configured,
    #: 'timezone' is mandatory.
    #: value: ranges from 0 to 100
    year_backups = resource.Body('year_backups', type=int)


class Properties(resource.Resource):
    #: Properties
    #: Scheduling rule.
    #: In the replication policy, you are advised  to set one time point
    #: for one day. A max number of 24 rules can be configured.
    #: The scheduling rule complies with iCalender RFC 2445, but ita supports
    #: only the following params: 'FREQ', 'BYDAY', 'BYHOUR', 'BYMINUTE', and
    #: 'INTERVAL.
    #: 'FREQ' can be set to 'WEEKLY' and 'DAILY'.
    #: 'BYDAY' can bne set to: MO, TU, WE, TH, FR, SA, SU
    #: 'BYHOUR' ranges from 0 to 23 hours
    #: 'BYMINUTE' ranges from 0 to 59
    #: The scheduling interval must not be less than 1 hour.
    #: A max of 24 time points are allowed in a day, e.g. If scheduling time
    #: is 14:00 from Monday to Sunday, set the scheduling rule as follows:
    #: 'FREQ=WEEKLY;BYDAY=MO;TU;WE;TH;FR;SA;SU;BYHOUR=14;BYMINUTE=00'
    #: If the scheduling time is 14:00 every day:
    #: FREQ=DAILY;INTERVAL=1;BYHOUR=14;BYMINUTE=00
    pattern = resource.Body('pattern', type=list)
    #: Start time of the scheduler in the format 2020-01-08 09:59:49
    start_time = resource.Body('start_time')


class Trigger(resource.Resource):
    #: Properties
    #: Scheduler attributes
    properties = resource.Body('properties', type=Properties)


class Vault(resource.Resource):
    #: Properties
    #: ID of the associated remote vault
    destination_vault_id = resource.Body('destination_vault_id')
    #: Vault ID
    vault_id = resource.Body('vault_id')


class Policy(resource.Resource):
    """CBR Policy Resource"""
    resource_key = 'policy'
    resources_key = 'policies'
    base_path = '/policies'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'operation_type', 'vault_id')

    #: Properties
    #: associated vault
    associated_vaults = resource.Body('associated_vaults', type=list,
                                      list_type=Vault)
    #: Whether to enable the policy
    enabled = resource.Body('enabled', type=bool)
    #: Policy ID
    id = resource.Body('id')
    #: Policy Name
    #: Max: 64 chars
    name = resource.Body('name')
    #: Scheduling parameter
    operation_definition = resource.Body('operation_definition',
                                         type=OperationDefinition)
    #: Policy type
    #: values: backup, replication
    operation_type = resource.Body('operation_type')
    #: Time rule for the policy execution
    trigger = resource.Body('trigger', type=Trigger)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

import openstack
import openstack.exceptions as ex

from otcextensions.tests.functional.sdk.cts import TestCts

_logger = openstack._log.setup_logging('openstack')


class TestTraces(TestCts):
    uuid_v4 = uuid.uuid4().hex[:8]
    tracker_name = 'system'
    tracker_prefix = f'{uuid_v4}-cts-sdk-'
    bucket_name = uuid_v4 + 'cts-sdk-test'

    def setUp(self):
        super(TestTraces, self).setUp()
        self.obs_client = self.conn.obs
        self.container = self.obs_client.create_container(
            name=self.bucket_name,
            storage_acl='public-read-write',
            storage_class='STANDARD'
        )
        try:
            self.tracker = self.client.create_tracker(
                bucket_name=self.bucket_name,
                file_prefix_name=self.tracker_prefix,
            )
        except (ex.BadRequestException, ex.HttpException):
            self.tracker = self.client.get_tracker(self.tracker_name)

    def tearDown(self):
        super(TestTraces, self).tearDown()
        try:
            if self.tracker:
                self.client.delete_tracker(self.tracker)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        objects = self.obs_client.objects(
            container=self.container
        )
        for obj in objects:
            self.obs_client.delete_object(obj, container=self.container)
        self.addCleanup(self.obs_client.delete_container, self.container)

    def test_list_traces(self):
        traces = self.client.traces()
        self.assertIsNotNone(traces)
        tr = next(traces)
        self.assertEqual(tr['tracker_name'], self.tracker_name)
        self.assertEqual(tr['service_type'], 'CTS')

    def test_get_tracker(self):
        tracker = self.client.get_tracker(self.tracker_name)
        self.assertEqual(tracker['file_prefix_name'], self.tracker_prefix)
        self.assertEqual(tracker['bucket_name'], self.bucket_name)

    def test_update_tracker(self):
        new_prefix = f'{self.uuid_v4}-cts-sdk-new-'
        tracker = self.client.get_tracker(self.tracker_name)
        self.assertEqual(tracker['file_prefix_name'], self.tracker_prefix)
        tracker = self.client.update_tracker(
            tracker=self.tracker_name,
            file_prefix_name=new_prefix,
            bucket_name=self.bucket_name
        )
        self.assertEqual(tracker['file_prefix_name'], new_prefix)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CES Alarm v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListMetricData(command.Lister):
    _description = _('List CES event data')
    columns = (
        'timestamp',
        'type',
        'value'
    )

    def get_parser(self, prog_name):
        parser = super(ListMetricData, self).get_parser(prog_name)
        parser.add_argument(
            '--namespace',
            metavar='<namespace>',
            required=True,
            help=_('Specifies the namespace of the metric such as:\n'
                   'SYS.ECS, SYS.AS')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            required=True,
            help=_('Specifies the event type such as:\n'
                   'instance_host_info')
        )
        parser.add_argument(
            '--dim',
            metavar='<key,value>',
            required=True,
            help=_('Specifies the monitoring dimension:\n'
                   'dim.0=instance_id,123-456-789')
        )
        parser.add_argument(
            '--time-from',
            metavar='<from>',
            required=True,
            help=_('UNIX timestamp in ms from which the data is '
                   'collected.')
        )
        parser.add_argument(
            '--time-to',
            metavar='<to>',
            required=True,
            help=_('UNIX timestamp in ms to which the data is '
                   'collected.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.ces

        query = {}
        query['namespace'] = parsed_args.namespace
        query['type'] = parsed_args.type
        query['dim.0'] = parsed_args.dim
        query['from'] = parsed_args.time_from
        query['to'] = parsed_args.time_to

        data = client.event_data(**query)

        table = (self.columns,
                 (utils.get_dict_properties(
                     s, self.columns
                 ) for s in data))
        return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

import mock
from openstackclient.tests.unit import utils as tests_utils
from otcextensions.common import cli_utils

from otcextensions.osclient.modelartsv2.v2 import dataset_import_task
from otcextensions.tests.unit.osclient.modelartsv2.v2 import fakes

_COLUMNS = (
    "annotated_sample_count",
    "created_at",
    "dataset_id",
    "elapsed_time",
    "finished_file_count",
    "finished_file_size",
    "id",
    "import_path",
    "import_type",
    "imported_sample_count",
    "imported_sub_sample_count",
    "status",
    "task_id",
    "total_file_count",
    "total_file_size",
    "total_sample_count",
    "total_sub_sample_count",
    "updated_at",
)


class TestListDatasetImportTask(fakes.TestModelartsv2):
    _dataset = fakes.FakeDataset.create_one()
    objects = fakes.FakeDatasetImportTask.create_multiple(3)

    column_list_headers = (
        "Task Id",
        "Created At",
        "Status",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.task_id,
                cli_utils.UnixTimestampFormatter(s.created_at),
                s.status,
            )
        )

    def setUp(self):
        super(TestListDatasetImportTask, self).setUp()

        self.cmd = dataset_import_task.ListDatasetImportTasks(self.app, None)

        self.client.find_dataset = mock.Mock(return_value=self._dataset)
        self.client.dataset_import_tasks = mock.Mock()
        self.client.api_mock = self.client.dataset_import_tasks

    def test_list(self):
        arglist = ["dataset-id"]

        verifylist = [
            ("dataset", "dataset-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_dataset.assert_called_with(
            "dataset-id", ignore_missing=False
        )
        self.client.api_mock.assert_called_with(self._dataset)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "dataset-id",
            "--limit",
            "1",
            "--offset",
            "2",
        ]

        verifylist = [
            ("dataset", "dataset-id"),
            ("limit", 1),
            ("offset", 2),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_dataset.assert_called_with(
            "dataset-id", ignore_missing=False
        )
        self.client.api_mock.assert_called_with(
            self._dataset, limit=1, offset=2
        )


class TestShowDatasetImportTask(fakes.TestModelartsv2):
    _dataset = fakes.FakeDataset.create_one()
    columns = _COLUMNS
    object = fakes.FakeDatasetImportTask.create_one()

    formatters = {
        "created_at": cli_utils.UnixTimestampFormatter,
        "updated_at": cli_utils.UnixTimestampFormatter,
    }
    data = fakes.gen_data(object, columns, formatters)

    def setUp(self):
        super(TestShowDatasetImportTask, self).setUp()

        self.cmd = dataset_import_task.ShowDatasetImportTask(self.app, None)

        self.client.find_dataset = mock.Mock(return_value=self._dataset)
        self.client.get_dataset_import_task = mock.Mock(
            return_value=self.object
        )

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = ["dataset-id", "task-id"]

        verifylist = [
            ("dataset", "dataset-id"),
            ("taskId", "task-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_dataset.assert_called_with(
            "dataset-id", ignore_missing=False
        )
        self.client.get_dataset_import_task.assert_called_with(
            self._dataset, "task-id"
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create CCE Cluster node
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


node = conn.create_cce_cluster_node(
    annotations={'annotation1': 'abc'},
    availability_zone='eu-de-01',
    cluster='d9b37539-c927-11eb-bc4e-0255ac101123',
    count=1,
    flavor='s2.large.2',
    k8s_tags={
        "muh": "kuh"},
    ssh_key='keypair-pub',
    labels={'foo': 'bar'},
    max_pods=16,
    name='node2',
    network='25d24fc8-d019-4a34-9fff-0a09fde6a123',
    offload_node=False,
    os='CentOS 7.7',
    root_volume_size=40,
    root_volume_type='SATA',
    tags=[
        {
            'key': 'hellokey1',
            'value': 'hellovalue1'
        }, {
            'key': 'hellokey2',
            'value': 'hellovalue2'
        }],
    data_volumes=[{
        'volumetype': 'SATA',
        'size': 110,
        'encrypted': False,
        'cmk_id': ''
    }, {
        'volumetype': 'SAS',
        'size': 120,
        'encrypted': False,
        'cmk_id': ''
    }],
    wait=False)
print(node)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class ListEvents(resource.Resource):
    id = resource.Body('id', type=str)
    last_modified = resource.Body('last_modified', type=int)
    name = resource.Body('name', type=str)


class Event(resource.Resource):
    base_path = '/fgs/functions/%(function_urn)s/events'
    resources_key = 'events'
    # Capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True
    allow_commit = True

    # Properties
    function_urn = resource.URI('function_urn', type=str)
    #: Test event name. Max. 25 of letters, digits, hyphens (-),
    #: and underscores (_). Start with a letter,
    #: and end with a letter or digit.
    name = resource.Body('name', type=str)
    #: Test event content, which is a Base64-encoded JSON character string.
    content = resource.Body('content', type=str)

    # Attributes
    #: Test event ID.
    id = resource.Body('id', type=str)
    #: Total number of test events.
    count = resource.Body('count', type=int)
    #: Next read location.
    next_marker = resource.Body('next_marker', type=int)
    #: Last update time.
    updated_at = resource.Body('last_modified', type=int)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.common import sdk_utils
from otcextensions.osclient.anti_ddos.v1 import config
from otcextensions.tests.unit.osclient.anti_ddos.v1 import fakes


class TestListConfig(fakes.TestAntiDDoS):

    objects = fakes.FakeConfig.create_multiple(3)

    columns = (
        'traffic_limited_list', 'http_limited_list', 'connection_limited_list'
    )

    data = []

    for s in objects:
        data.append((
            sdk_utils.ListOfDictColumn(s.traffic_limited_list),
            sdk_utils.ListOfDictColumn(s.http_limited_list),
            sdk_utils.ListOfDictColumn(s.connection_limited_list),
        ))

    def setUp(self):
        super(TestListConfig, self).setUp()

        self.cmd = config.ListConfig(self.app, None)

        self.client.configs = mock.Mock()

    def test_list(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.configs.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.configs.assert_called_once_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))
